/* Fetch_Settings.c

	Simple dialog routines to handle receiving input of a user's account name
	and password, which are then placed in the user's PPP connect settings,
	used to log onto the service.
	
	The password and account name are entered into the connect script for the
	active config or into the Authentication settings for the active config, as
	determined by settings pulled from the 'ISPc' resource ID#3000 in the resource
	file.
		
	See #define STANDALONE in the PPP_Prefs.h to change from compiling as a standalone
	application to compiling as a code resource of type 'IEnd'.
*/

#include "PPP-Prefs.h"
#include "PrefSaver Typedefs.h"

#define		kMoveToFront		(WindowPtr)-1L
#define		kDisable			255
#define		kEnable				0

#define		kReturnKey			0x0D
#define		kEnterKey			0x03
#define		kESCKey				0x1B
#define		kTildeKey			0x7E

#define		kCtrlCharEnd		0x1F
#define		kDeleteKey			0x7F

	// numeric chars top/bottom limits
#define		kNumeralZero		0x30
#define		kNumeralNine		0x39

	// Lowercase alphabetic chars top/bottom limits
#define		kSmall_a			0x61
#define		kSmall_z			0x7A
	// Uppercase letters will be converted to lowercase
#define		kBig_A				0x41
#define		kBig_Z				0x5A

#define		kUpperToLowerCaseOffset	0x20

#define 	kMyButtonDelay 		8
#define		kSimpleModalFilter	(ProcPtr)&SimpleModalFilter

#define		rUserPrefsDialog		3000
#define			defOutline		3
#define			iUserName		5
#define			iPassword		6
#define			iDemoOnlyStr	14

#define		rConfirmSettingsAlrt	5001
#define		eISP_SetupErr			5002


/* Prototypes */
Boolean			GetUserPrefs( struct ppp_config	*theConfig, OSErr *err );
pascal Boolean	SimpleModalFilter( DialogPtr dialog, EventRecord *event,
									short *itemHit );
pascal void		buttonProc( DialogPtr theDialog, short theItem);
OSErr			GetISP_Info( PrefDialogPtr prefDialog, Boolean *acceptable );
void 			Check_StringLenLimit( short iHit, DialogPtr dialog, EventRecord *event);
void			Check_IllegalCharEntry( short iHit, EventRecord *event,
														PrefDialogPtr prefDialog);


/*************************** GetUserPrefs *********************************/

Boolean		GetUserPrefs( struct ppp_config	*theConfig, OSErr *err )
{
	DialogPtr		dialog;
	Ptr				prefDialog;
	GrafPtr			savePort;
	Boolean			dialogDone = false, saveConfig = false, acceptable;
	short			iType, iHit;
	Handle			iHandle, nameHandle, pwordHandle;
	ControlHandle	okButton;
	Rect			iRect;
	Str255			theString, the_otherString;
	Point			topCorner;
	
	prefDialog = NewPtrClear(sizeof(PrefDialog));
	if (! prefDialog){
		*err = MemError();
		goto getOut;
	}
	
	*err = GetISP_Info( prefDialog, &acceptable);
	if (*err || (!acceptable)) { goto getOut; }	// Failed reading ISP setup info
	
	GetPort(&savePort);
	dialog = GetNewDialog(rUserPrefsDialog, (DialogPeek)prefDialog, kMoveToFront);
	*err = LMGetResErr();
	if (!dialog)	{ goto getOut; }
	
	SetWRefCon(dialog, (long)GetUserPrefs);	
	SetPort(dialog);
		
	GetDItem(dialog, defOutline, &iType, &iHandle, &iRect);
	SetDItem(dialog, defOutline, iType, buttonProc, &iRect);
	
		// Disable "OK" until edit fields are filled in
	GetDItem(dialog, ok, &iType, &okButton, &iRect);
	HiliteControl(okButton, kDisable);
	
	GetDItem(dialog, iUserName, &iType, &nameHandle, &iRect);
	GetDItem(dialog, iPassword, &iType, &pwordHandle, &iRect);
	
		// If this is not a demo, hide the "THIS IS A DEMO" message
	if (! ((PrefDialogPtr)prefDialog)->ISPspec.thisIsADemo)
		HideDItem( dialog, iDemoOnlyStr);
	
	ShowWindow(dialog);
	
	while (! dialogDone)
	{
		ModalDialog(kSimpleModalFilter, &iHit);
		
		switch( iHit)
		{
			case ok:
				HideWindow(dialog);
				GetIText(nameHandle, theString);
				GetIText(pwordHandle, the_otherString);
				ParamText(theString, the_otherString, 0,0);
				iHit = Alert(rConfirmSettingsAlrt,kSimpleModalFilter);
				if (iHit == ok)
					dialogDone = true;
				else
					ShowWindow(dialog);
				break;
			case cancel:
				dialogDone = true;
				break;
			default:
				GetIText(nameHandle, theString);
				if (theString[0])
					GetIText(pwordHandle, theString);
				
				if (! theString[0]){
					if ((*okButton)->contrlHilite == kEnable)
						HiliteControl(okButton, kDisable);
				}
				else{
					if ((*okButton)->contrlHilite == kDisable)
						HiliteControl(okButton, kEnable);
				}
				break;
		}
	}
	
	if (iHit == ok){
		if (! ((PrefDialogPtr)dialog)->ISPspec.thisIsADemo){ // If this is a demo, don't save anything
		
		/* Write the user's settings into the appropriate fields */
			GetIText(nameHandle, theString);
			if (((PrefDialogPtr)dialog)->ISPspec.useConnectScript)
				BlockMove(theString, &(theConfig->commands[((PrefDialogPtr)dialog)->ISPspec.accountNameField].scriptstr), theString[0]+1);
			else	// Write into Authentication settings instead
				BlockMove(theString, &(theConfig->defaultid), theString[0]+1);
			
			GetIText(pwordHandle, theString);
			if (((PrefDialogPtr)dialog)->ISPspec.useConnectScript)
				BlockMove(theString, &(theConfig->commands[((PrefDialogPtr)dialog)->ISPspec.passwordField].scriptstr), theString[0]+1);
			else	// Write into Authentication settings instead
				BlockMove(theString, &(theConfig->defaultpw), theString[0]+1);
	
			saveConfig = true;
		}
	}
	
	SetPort(savePort);
	DisposDialog(dialog);
	
getOut:
	return (saveConfig);
}

	
/************************* SimpleModalFilter ********************************/

pascal Boolean	SimpleModalFilter( DialogPtr dialog, EventRecord *event,
									short *itemHit )
{
	Boolean 		returnVal = false;
	WindowPtr		temp;
	ProcPtr			*standardProc;
	short			iType, iHit, charLimit;
	Handle			iHandle;
	Rect			iRect;
	char			theChar;
	long			waitTicks;
			
	/* This filter is used to queue the ok/cancel buttons via the keyboard
		while remaining compatible with System 6.  It also limits the user
		to entering no more than 8 alphanumeric characters in the edit text fields	*/
		 
	if((event->what == keyDown) || (event->what == autoKey))
	{
		theChar = (event->message & charCodeMask);
		switch (theChar)
		{
			case kReturnKey:
			case kEnterKey:
					/* change whatever is the current item to the OK item */
				GetDItem(dialog,OK,&iType,&iHandle,&iRect);
				if ((**(ControlHandle)iHandle).contrlHilite != kDisable){
					*itemHit = OK;
					HiliteControl((ControlHandle)iHandle,inButton);
					Delay(kMyButtonDelay,&waitTicks);	/* wait about 8 ticks */
					HiliteControl((ControlHandle)iHandle,false);
					returnVal = true;
				}
				else
					event->what = nullEvent;
				break;
			case kESCKey:
			case kTildeKey:
				*itemHit = cancel;
				GetDItem(dialog,cancel,&iType,&iHandle,&iRect);
				HiliteControl((ControlHandle)iHandle,inButton);
				Delay(kMyButtonDelay,&waitTicks);
				HiliteControl((ControlHandle)iHandle,false);
				returnVal = true;
				break;
			default:
				if (GetWRefCon(dialog) == (long)GetUserPrefs){	// Ignore the confirm alert
					
						// Disallow command keys (prevent Edit commands) and ctrl keys (bad chars)
					if ( ((event->modifiers & cmdKey) != 0) || ((event->modifiers & controlKey) != 0)){
						event->what = nullEvent;
						SysBeep(10);
					}
					
						// Check non-ctrl chars against field length limit
					if (((unsigned short)theChar > kCtrlCharEnd) && (theChar != kDeleteKey)){
						iHit = ((DialogPeek)dialog)->editField + 1;
						Check_StringLenLimit(iHit, dialog, event);	// Set event to null if too many chars
						
							//  Check for illegal char entry
						Check_IllegalCharEntry(iHit, event, (PrefDialogPtr)dialog);
					}
				}
				break;
		}
	}
	return (returnVal);
}


/********************* buttonProc **************************/

pascal void		buttonProc( DialogPtr theDialog, short theItem)	// Outline Default Button
{
	short	type;
	Rect	box;
	Handle	itemHdl;

	GetDItem(theDialog, ok, &type, &itemHdl, &box);

	PenSize(3,3);
	InsetRect( &box, -4, -4);
	FrameRoundRect( &box, 16, 16);
	PenSize(1,1);
}


/************************ GetISP_Info ********************************/

OSErr		GetISP_Info( PrefDialogPtr prefDialog, Boolean *acceptable )
{
	ISP_SpecHandle	resHandle;
	OSErr			err = 0;
	
	*acceptable = true; // assume settings are reasonable
	
	resHandle = (Handle)GetResource( 'ISPc', 3000);
	if (! resHandle){
		err = ResError();
		if (! err)
			err = resNotFound;
	}
	
	if (!err){
		prefDialog->ISPspec = **resHandle;
		ReleaseResource((Handle)resHandle);
		
		// Verify acceptability of settings
			// Make sure field lengths settings are acceptable
		if ( (prefDialog->ISPspec.acctNameLen > MAXSLEN || prefDialog->ISPspec.passwordLen > MAXSLEN)
			|| (prefDialog->ISPspec.acctNameLen < 1 || prefDialog->ISPspec.passwordLen < 1) )
			// The length setting for one or both text fields is unacceptable
			*acceptable = false;
			
			// If using connect script, make sure field number assignments are reasonable
		if (prefDialog->ISPspec.useConnectScript){
			if ( (prefDialog->ISPspec.accountNameField > 7 || prefDialog->ISPspec.passwordField > 7)
				|| (prefDialog->ISPspec.accountNameField < 0 || prefDialog->ISPspec.passwordField < 0)
				|| (prefDialog->ISPspec.accountNameField == prefDialog->ISPspec.passwordField) )
				// The field number for one or both connect script field selections
					// Is unacceptable
			*acceptable = false;
		}
		if (! *acceptable)
			StopAlert(eISP_SetupErr, nil);	// Bad configurartion of 'ISPc'
	}
	
	return (err);
}


/********************** Check_StringLenLimit *****************************/

void Check_StringLenLimit( short iHit, DialogPtr dialog, EventRecord *event)
{
	short	charLimit, charsHilited, resultStrLen;
	Boolean	nullTheEvent = false;
	short	iType;
	Rect	iRect;
	Handle	iHandle;
	Str255	iString;
	
	if (iHit == iUserName)
		charLimit = ((PrefDialogPtr)dialog)->ISPspec.acctNameLen;
	else
		charLimit = ((PrefDialogPtr)dialog)->ISPspec.passwordLen;
		
		// If char is added, will length exceed limit?
	GetDItem(dialog, iHit, &iType, &iHandle, &iRect);
	GetIText(iHandle, iString);
	charsHilited = (*((DialogPeek)dialog)->textH)->selEnd -
							(*((DialogPeek)dialog)->textH)->selStart;
	resultStrLen = (iString[0] + 1) - charsHilited;
	if (resultStrLen > charLimit){	
		event->what = nullEvent;
		SysBeep(10);
	}
}


/************************** Check_IllegalCharEntry ****************************/

void		Check_IllegalCharEntry( short iHit, EventRecord *event,
												PrefDialogPtr prefDialog)
{
	char		theChar;
	Boolean		match = false;	// Assume match not found in additional accepted chars
	StringPtr	acceptableChars;
	short		len, loop;
	
	//  Check for illegal char entry
	theChar = (event->message & charCodeMask);
	
	if (theChar < kSmall_a || theChar > kSmall_z){
		if (theChar < kNumeralZero || theChar > kNumeralNine){
				// Check if need we to convert uppercase to lowercase
			if (theChar >= kBig_A && theChar <= kBig_Z){
				if ( (iHit == iUserName && (! prefDialog->ISPspec.acctName_AllowCaps))
							|| (iHit == iPassword && (! prefDialog->ISPspec.pWord_AllowCaps)) ){
					if ((! prefDialog->ISPspec.acctName_AllowCaps) &&
								(! prefDialog->ISPspec.pWord_AllowCaps))
						event->message += kUpperToLowerCaseOffset;
				}
			}
			else{	// The char isn't a letter or number
					// Check for additional acceptable chars
				if (iHit == iUserName)
					acceptableChars = &(prefDialog->ISPspec.acctName_OtherCharStr);
				else
					acceptableChars = &(prefDialog->ISPspec.pWord_OtherCharStr);
				len = acceptableChars[0];
				if (len){
					loop = 1;
					while (loop <= len){	// Search for the char in string
						if (theChar == acceptableChars[loop]){
							match = true;
							loop = len+1;
						}
						else
							loop++;
					}
				}
				if (!match){	// character is unacceptable
					event->what = nullEvent;
					SysBeep(10);
				}
			}
		}
	}
}